/*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2015 Camil Staps <info@camilstaps.nl>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.camilstaps.route66;

import java.util.Random;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Route66 Controller
 * 
 * Use different threads for every driver
 * 
 * @author Pieter Koopman, Camil Staps
 */
public class Controller {
    private int delay = 120;                // average sleep time
    private final Model model;              // the model
    private final Random random;            // a random generator

    /**
     * The constructor of the controller
     * @param model holds the cars
     */
    public Controller(Model model) {
        this.model  = model;
        random      = new Random();
    }

    /**
     * Tell the drivers of all cars in the model to start driving
     */
    public void run() {
        ExecutorService service = Executors.newCachedThreadPool();
        for (Car car : model.getCars()) {
            Driver d = car.getDriver();
            d.setDelay(delay);
            service.execute(d);
        }
    }
    
    /**
     * Stop all cars
     */
    public void stopCars() {
        for (Car car : model.getCars()) {
            car.getDriver().stopRunning();
        }
    }

    /**
     * Resume all cars
     */
    public void resumeCars() {
        for (Car car : model.getCars()) {
            car.getDriver().resumeRunning();
        }
    }
    
    public int getDelay() {
        return delay;
    }
    
    /**
     * Set delay between maximum and minimum bounds
     * @param d 
     */
    public void setDelay(int d) {
        delay = Math.max(20, Math.min (2000, d));
        for (Car car : model.getCars()) {
            car.getDriver().setDelay(delay);
        }
    }
}
