/*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2015 Camil Staps <info@camilstaps.nl>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.camilstaps.route66;

import java.util.Random;

/**
 * A Driver takes care its car doesn't collide
 * 
 * @author Camil Staps
 */
public class Driver implements Runnable {
    
    private final Car car;
    private final Model model;
    
    private final Random r;
    
    private boolean running = false;
    private int delay;
    
    public Driver(Car car, Model model) {
        this.car = car;
        this.model = model;
        r = new Random();
    }
    
    /**
     * Stop running
     */
    public void stopRunning() {
        running = false;
    }
    
    /**
     * Resume running
     */
    public void resumeRunning() {
        if (running == false) {
            synchronized (this) {
                running = true;
                notifyAll();
            }
        }
    }
    
    /**
     * Set some approximate delay for {@link Driver#pause()}. The actual delay
     * is still pseudorandom.
     * @param value 
     */
    public void setDelay(int value) {
        delay = value;
    }

    @Override
    /**
     * Forever wait for safety, step and pause for a pseudorandom time.
     */
    public synchronized void run() {
        running = true;
        while (true) {
            do {
                while (!running) {
                    try {
                        wait();
                    } catch (InterruptedException ex) {
                    }
                }
            } while (!model.isSafeLocation(car, car.getNewLocation()));

            car.step();
            
            pause();
        }
    }
    
    /**
     * Pause for some pseudorandom time
     */
    protected void pause() {
        try {
            int bonus = 0;
            if (Math.abs((System.currentTimeMillis() / 1000) % Model.NUMBEROFCARS - car.getNumber()) < 2) {
                bonus = -50;
            }
            Thread.sleep(Math.abs(r.nextInt(100) + delay + bonus));
        } catch (InterruptedException ex) {}
    }
    
}
