/*
 * The MIT License (MIT)
 * 
 * Copyright (c) 2015 Camil Staps <info@camilstaps.nl>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.camilstaps.route66;

import java.awt.Color;
import java.awt.Graphics;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * OO13route66 animation
 *
 * Yields a graphical view on all cars
 * 
 * @author Pieter Koopman, Camil Staps
 */
public class RoadView extends JFrame
{
    JPanel panel;                               // the panel to draw the cars
    public static final int WINDOWSIZE = 600;   // the window size
    private final Model model;                  // the model knows the position of cars

    /**
     * A subclass of JPanel to draw the cars.
     * This is a bit of a hack to ensure that this panel is painted correctly.
     */
    private class CarPanel extends JPanel {
        /**
         * Constructor: just call the constructor of the base class
         */
        public CarPanel() {
            super();
        }
        /**
         * paint the roads and cars
         * @param g Graphics to paint on
         */
        @Override
        public void paint (Graphics g) {
            paintRoad(g);
            paintCars(g);
        }
    }
    /**
     * The constructor of RoadView
     * @param model
     */
    RoadView(Model model) {
        super("Route66 traffic simulator");
        this.model = model;
        setSize(WINDOWSIZE, WINDOWSIZE);
        setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        panel = new CarPanel();
        add(panel);
    }            
    /**
     * Paint the view of the model graphically
     * @param g
     */
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        panel.repaint();
    }
    /**
     * Paint all cars
     * @param g graphics to draw on
     */
    private void paintCars (Graphics g) {
        for (int i = 0; i < Model.NUMBEROFCARS; i += 1) {
            model.getCar(i).paint(g);
        }
    }

    /**
     * Paint the roads
     * @param g graphics to draw on
     */
    private void paintRoad (Graphics g) {
        final int left = getStartCrossing();
        final int width = getEndCrossing() - left;
        g.setColor(Color.white);    // background
        g.fillRect(0, 0, WINDOWSIZE, WINDOWSIZE);
        g.setColor(Color.darkGray); // streets
        g.fillRect(0, left, WINDOWSIZE, width);
        if (Model.DIRECTIONS > 2) {  // paint a crossing if there are 4 directions
            g.fillRect(left, 0, width, WINDOWSIZE);
        }
    }
    
    /**
     * Get the relevant half of the coordinate where the crossing starts
     * @return 
     */
    public static int getStartCrossing() {
        return (WINDOWSIZE / 2) - Car.CARWIDTH - 4;
    }
    
    /**
     * Get the relevant half of the coordinate where the crossing ends
     * @return 
     */
    public static int getEndCrossing() {
        return getStartCrossing() + 2 * Car.CARWIDTH + 8;
    }
}
